/*
 * Copyright (c) Camden Dixie O'Brien
 * SPDX-License-Identifier: AGPL-3.0-only
 */

#include <X11/Xlib.h>
#include <assert.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#define MAZE_SIZE 24

#define GRID_SIZE (2 * MAZE_SIZE - 1)
#define MARGIN 2
#define WALL_THICKNESS 1
#define WINDOW_SIZE (GRID_SIZE + 2 * (WALL_THICKNESS + MARGIN))

#define PX(x) ((x) << 3)

typedef struct {
	bool cells[GRID_SIZE][GRID_SIZE];
} maze_t;

static void draw_maze(Display *dpy, Window w, GC gc, const maze_t *m)
{
	// Draw walls
	XFillRectangle(
	    dpy, w, gc, PX(MARGIN), PX(MARGIN), PX(GRID_SIZE + 2),
	    PX(WALL_THICKNESS));
	XFillRectangle(
	    dpy, w, gc, PX(MARGIN), PX(MARGIN + WALL_THICKNESS + GRID_SIZE),
	    PX(GRID_SIZE + 2), PX(WALL_THICKNESS));
	XFillRectangle(
	    dpy, w, gc, PX(MARGIN), PX(MARGIN + WALL_THICKNESS + 1),
	    PX(WALL_THICKNESS), PX(GRID_SIZE - 1));
	XFillRectangle(
	    dpy, w, gc, PX(MARGIN + WALL_THICKNESS + GRID_SIZE),
	    PX(MARGIN + WALL_THICKNESS), PX(WALL_THICKNESS), PX(GRID_SIZE));

	// Draw cells
	for (int x = 0; x < GRID_SIZE; ++x) {
		for (int y = 0; y < GRID_SIZE; ++y) {
			if (m->cells[x][y])
				continue;
			const int left = PX(MARGIN + WALL_THICKNESS + x);
			const int top = PX(MARGIN + WALL_THICKNESS + y);
			XFillRectangle(dpy, w, gc, left, top, PX(1), PX(1));
		}
	}

	XFlush(dpy);
}

int main(void)
{
	XEvent evt;
	Display *dpy = XOpenDisplay(NULL);
	assert(dpy);

	// Create window and configure graphics context
	const int black = BlackPixel(dpy, DefaultScreen(dpy));
	const int white = WhitePixel(dpy, DefaultScreen(dpy));
	Window w = XCreateSimpleWindow(
	    dpy, DefaultRootWindow(dpy), 0, 0, PX(WINDOW_SIZE), PX(WINDOW_SIZE),
	    0, white, white);
	Atom del = XInternAtom(dpy, "WM_DELETE_WINDOW", false);
	XSetWMProtocols(dpy, w, &del, 1);
	GC gc = DefaultGC(dpy, DefaultScreen(dpy));
	XSetForeground(dpy, gc, black);

	// Map window
	XSelectInput(dpy, w, StructureNotifyMask);
	XMapWindow(dpy, w);
	do
		XNextEvent(dpy, &evt);
	while (MapNotify != evt.type);

	maze_t m;
	memset(&m, 0, sizeof(maze_t));
	draw_maze(dpy, w, gc, &m);

	// Wait for window exit
	bool is_del = false;
	do {
		XNextEvent(dpy, &evt);
		if (ClientMessage == evt.type)
			is_del = (unsigned long)evt.xclient.data.l[0] == del;
	} while (!is_del);

	XCloseDisplay(dpy);
	return EXIT_SUCCESS;
}
